package packageForL9L10;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;

/**
 * <p>
 * The class encapsulates a view of a SimpleThreeDotModel (where a "view" is a
 * visual representation of data). This view is based on a widget container and
 * can be instantiated and displayed.
 * </p>
 * 
 * <p>
 * The view consists of a single widget which provides a single bounded
 * two-dimensional plane. The dots in the data model are each drawn on the plane
 * in their specified location and with their specified size. The user space is
 * mapped directly to the device space.
 * </p>
 * 
 * <p>
 * The only constructor this class has is one that requires a
 * SimpleThreeDotModel object. It doesn't make sense to create this view without
 * its data model.
 * </p>
 * 
 * <p>
 * This view is capable of listening to data models and will install itself on
 * the passed data model. Any time the data model changes, this view will
 * repaint itself.
 * </p>
 * 
 * @author mb
 * 
 */
public class DotFrame extends JFrame implements DotModelListener {
	private SimpleThreeDotModel_v2 theModel;
	private int width = 400;
	private int height = 400;

	private JPanel thePanel;

	private DotFrame() {

	}

	/**
	 * Constructs a view as specified in the class description. The view gets
	 * installed upon the passed model.
	 * 
	 * @param model
	 */
	public DotFrame(SimpleThreeDotModel_v2 model) {
		int width = 400;
		int height = 400;
		this.setTitle("My Frame");
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(width, height);
		this.setMinimumSize(new Dimension(width, height));
		thePanel = new DotPanel(model);
		this.add(thePanel);

		this.pack();
		this.setVisible(true);

		theModel = model;
		// does this view need to update itself in response to any changes
		// in the model? If so, install a listener here
		model.addListener(this);

		/**
		 * ref p.99 At this point, the JPanel does not have knowledge about the
		 * widgets that have been constructed above. The above widgets are not
		 * added into the widget tree (see p. 17, fig 2.3) unless the add
		 * operation has been specifically invoked. Recall that membership in
		 * the window's widget tree allows the widget to be drawn and to receive
		 * events.
		 * 
		 * So we add each of the widgets to this panel's widget tree
		 */
		// add widgets here
		// this.repaint();
		// this.invalidate();
	}

	/**
	 * This method renders the graphics of this view.
	 * 
	 * The method is invoked any time the view is repainted. Here is what the
	 * method actually does. We can see that it causes the whole view to be
	 * drawn anew. (Obviously, this is breaking the encapsulation, but is
	 * included for pedagogical reasons). <br />
	 * <code>
	 *  Graphics2D g2 = (Graphics2D) g;<br />
	 *  g2.clearRect(0, 0, width, height);<br />
	 * 	g2.setColor(theModel.getColor1());<br />
	 * 	g2.draw(theModel.getDot1());<br />
	 * 	g2.setColor(theModel.getColour1());<br />
	 * 	g2.draw(theModel.getDot2());<br />
	 * 	g2.setColor(theModel.getColor3());<br />
	 * 	g2.draw(theModel.getDot3());<br />
	 * 
	 * </code>
	 */
	public void paintComponent(Graphics g) {
		// super.paintComponent(g); // yes, this should be the first statement,
		// but I want some parallelism with Version 5

		Graphics2D g2 = (Graphics2D) g;
		g2.clearRect(0, 0, width, height); // this clears the
		// drawing space
		g2.setColor(theModel.getColour1());
		g2.draw(theModel.getDot1());
		g2.setColor(theModel.getColour2());
		g2.draw(theModel.getDot2());
		g2.setColor(theModel.getColour3());
		g2.draw(theModel.getDot3());
	}

	public Dimension getPreferredSize() {
		return new Dimension(width, height);
	}

	/**
	 * <p>
	 * Method that changes this view in response to any change in the data
	 * model. If the data model changes, the view repaints itself anew.
	 * <p/>
	 * 
	 * <p>
	 * This method is invoked to signal to this listener that the model (upon
	 * which this listener has been installed) has been changed. This provides
	 * an opportunity for the listener to something in response to any change to
	 * the model.
	 * </p>
	 * 
	 * <p>
	 * <b>Who invokes this method?</b> Think of who has been given access to
	 * this view... The documentation for the constructor specifies that this
	 * view has been installed upon the specified data model. So the data model
	 * is able to invoke this method within its inner workings.
	 * </p>
	 */
	public void changed() {
		// what should happen if we come to learn that the model has changed? we
		// should repaint ourself.
		this.repaint();
	}

}
